/*
 * Copyright (c) 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * UDP via the PXE
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <oskit/error.h>

#include "boot.h"
#include "pxe.h"
#include "decls.h"

int
udp_open(void)
{
	t_PXENV_UDP_OPEN	udp_open_params;

	memset(&udp_open_params, 0, sizeof(udp_open_params));
	udp_open_params.src_ip = IP4(&myip);

	pxe_invoke(PXENV_UDP_OPEN, &udp_open_params);
	if (udp_open_params.Status) {
		printf(__FUNCTION__ ": failed. 0x%x\n",
		       udp_open_params.Status);
		return OSKIT_E_FAIL;
	}
	
	return 0;
}

int
udp_close(void)
{
	t_PXENV_UDP_CLOSE	udp_close_params;

	memset(&udp_close_params, 0, sizeof(udp_close_params));

	pxe_invoke(PXENV_UDP_CLOSE, &udp_close_params);
	if (udp_close_params.Status) {
		printf(__FUNCTION__ ": failed. 0x%x\n",
		       udp_close_params.Status);
		return OSKIT_E_FAIL;
	}
	
	return 0;
}

int
udp_read(struct in_addr *dst_ip, int dst_port, void *buffer, int bufsize, 
	 int *out_actual, int *src_port, struct in_addr *src_ip)
{
	t_PXENV_UDP_READ	udp_read_params;

	memset(&udp_read_params, 0, sizeof(udp_read_params));
	udp_read_params.dest_ip     = IP4(dst_ip);
	udp_read_params.d_port      = htons(dst_port);
	udp_read_params.buffer_size = bufsize;
	udp_read_params.buffer.segment = (oskit_u16_t) 0;
	udp_read_params.buffer.offset  = (oskit_u16_t) (buffer);
	
	pxe_invoke(PXENV_UDP_READ, &udp_read_params);
	if (udp_read_params.Status)
		return OSKIT_E_FAIL;

	src_ip->s_addr = ((struct in_addr *) &udp_read_params.src_ip)->s_addr;
	*src_port      = ntohs(udp_read_params.s_port);
	*out_actual    = udp_read_params.buffer_size;

	return 0;
}

int
udp_write(struct in_addr *dst_ip,
	  int src_port, int dst_port, void *buffer, int bufsize)
{
	t_PXENV_UDP_WRITE	udp_write_params;

	memset(&udp_write_params, 0, sizeof(udp_write_params));
	udp_write_params.ip       = IP4(dst_ip);
	udp_write_params.gw       = IP4(&gatewayip);
	udp_write_params.src_port = htons(src_port);
	udp_write_params.dst_port = htons(dst_port);
	udp_write_params.buffer_size = bufsize;
	udp_write_params.buffer.segment = (oskit_u16_t) 0;
	udp_write_params.buffer.offset  = (oskit_u16_t) (buffer);
	
	pxe_invoke(PXENV_UDP_WRITE, &udp_write_params);
	if (udp_write_params.Status) {
		printf(__FUNCTION__ ": failed. 0x%x\n",
		       udp_write_params.Status);
		return OSKIT_E_FAIL;
	}

	return 0;
}
